//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormButtonContainer.h"
#include "ProgrammableButton.h"
#include "FormButtonEdit.h"
#include "Settings.h"
#include "UaMain.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmButtonContainer *frmButtonContainer;

//---------------------------------------------------------------------------
__fastcall TfrmButtonContainer::TfrmButtonContainer(TComponent* Owner, int width, int height,
    int startBtnId, int btnCnt,
	CallbackClick callbackClick)
	: TForm(Owner), startBtnId(startBtnId), btnCnt(btnCnt), callbackClick(callbackClick)
{
	assert(startBtnId >= 0);
	assert(btnCnt >= 0);
	assert(callbackClick);
	if (width > 0)
	{
		this->Width = width;
	}
	if (height > 0)
	{
    	this->Height = height;
	}
	for (int i=0; i<btnCnt; i++)
	{
		TProgrammableButton *panel = new TProgrammableButton(flowPanel, imgList);
		panel->Tag = i;
		panel->Parent = flowPanel;
		panel->Width = flowPanel->Width-2;
		//panel->AlignWithMargins = true;
		if (i < appSettings.btnConf.size())
		{
			ButtonConf &cfg = appSettings.btnConf[startBtnId + i];
			panel->SetConfig(cfg);
		}
		panel->OnClick = SpeedDialPanelClick;
        panel->PopupMenu = popupPanel;
		panel->UpdateCallbacks();
		panel->Visible = True;
		vpanels.push_back(panel);
	}
}
//---------------------------------------------------------------------------

void __fastcall TfrmButtonContainer::SpeedDialPanelClick(TObject *Sender)
{
	TProgrammableButton* panel = dynamic_cast<TProgrammableButton*>(Sender);
	if (panel == NULL)
	{
		TComponent *component = dynamic_cast<TComponent*>(Sender);
		assert(component);
		panel = dynamic_cast<TProgrammableButton*>(component->Owner);
	}
	assert(panel);
	int id = startBtnId + panel->Tag;

	ButtonConf &cfg = appSettings.btnConf[id];
	if (cfg.type == Button::HOLD)
	{
    	panel->SetDown(!panel->GetDown());
	}

	callbackClick(id, panel->GetDown());
}
//---------------------------------------------------------------------------

void __fastcall TfrmButtonContainer::miEditSpeedDialClick(TObject *Sender)
{
	TProgrammableButton* panel = dynamic_cast<TProgrammableButton*>(popupPanel->PopupComponent);
	assert(panel);
	int id = panel->Tag;

	EditContact(id);
}
//---------------------------------------------------------------------------

void TfrmButtonContainer::EditContact(int id)
{
	ButtonConf cfg = appSettings.btnConf[startBtnId + id];	// copy
	frmButtonEdit->ShowModal(&cfg);

	if (frmButtonEdit->isConfirmed())
	{
		if (cfg != appSettings.btnConf[startBtnId + id])
		{
			bool restartUa = false;
			if (cfg.type == Button::BLF || appSettings.btnConf[startBtnId + id].type == Button::BLF) {
            	restartUa = true;
			}
			
			appSettings.btnConf[startBtnId + id] = cfg;
			TProgrammableButton* panel = vpanels[id];
			panel->SetConfig(cfg);
			if (cfg.type != Button::BLF)
			{
				panel->SetState(DIALOG_INFO_UNKNOWN);	// make sure BLF icon is cleared
			}
			AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".json" );
			appSettings.Write(asConfigFile);

			if (restartUa)
			{
                appSettings.UpdateContacts();
				Ua::Instance().Restart();
			}
		}
	}
}

void TfrmButtonContainer::UpdateDlgInfoState(int id, int state)
{
	id -= startBtnId;
	if (id < 0 || id >= vpanels.size())
		return;	
	vpanels[id]->SetState((enum dialog_info_status)state);
}

void TfrmButtonContainer::UpdateHoldState(bool state)
{
	for (unsigned int i=0; i<vpanels.size(); i++)
	{
		ButtonConf &cfg = appSettings.btnConf[i + startBtnId];
		if (cfg.type == Button::HOLD)
		{
			vpanels[i]->SetDown(state);
		}
	}
}

